#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <vd2/VDXFrame/VideoFilter.h>
#include <vd2/VDXFrame/VideoFilterDialog.h>
#include "../resource.h"
#include "f_dlone_extras.h"

extern int g_VFVAPIVersion;

	//get the current module handle without worrying about the name
	//re http://blogs.msdn.com/b/oldnewthing/archive/2004/10/25/247180.aspx
	EXTERN_C IMAGE_DOS_HEADER __ImageBase;
	#define HINST_THISCOMPONENT ((HINSTANCE)&__ImageBase)


	const char* ComboTxtEffDisabled="Disable Effect";
	const char* ComboTxtEffEnabled="Show Effect";
	const char* ComboTxtEffHalf="Half Frame";

	const char* txtPreEffectOn="Pre-Effect Reduction";
	const char* txtPreEffectOff="Post-Effect Reduction";

//constants in order of descriptions above
	#define cEffectDisabled 0
	#define cEffectEnabled 1
	#define cEffectHalf 2

#define MyOptHiWidth_default 8
#define MyOptHiReduction_default 0
#define MyOptPreLevel_default 255


	HICON iconReset=0;

	const char* helpText="Brightness Type:\n" \
												"  Different methods of adjusting brightness\n" \
												"  Some brighten shadows more, etc...\n" \
												"\n" \
												"Brightness, Contrast, Saturation, Hue\n" \
												"  How much to alter the brightness\n" \
												"  How much to alter the contrast\n" \
												"  How colourful the image is\n" \
												"  Rotate the colours\n" \
												"\n" \
												"Darken Output:\n" \
												"  Reduces input vales from zero to full\n" \
												"  Darkens the entire video by a fixed amount before processing\n" \
												"  Useful if the video is too bright overall\n" \
												"\n" \
												"Hilight Reduction:\n" \
												"  Pre-Effect: Reduce hilights before or after applying effects\n" \
												"  Wide->Narrow: Left, adjust a wide range. Right, adjust a narrow range\n" \
												"  Amount: How much hilight reduction (far left= no reduction)\n" \
												"\n" \
												"Link Contrast With Saturation:\n" \
												"  When the contrast is altered, the saturation is modified as well\n" \
												"\n" \
												"Input Levels 16-239:\n" \
												"  Tick if the input video is in 16-239 format (default),\n" \
												"  or untick if the input video is in 0-255 format\n" \
												"\n" \
												"Output Levels 16-239:\n" \
												"  Tick for the output to be in 16-239 format (default)\n" \
												"\n" \
												"Preview Type:\n" \
												"  Show Effect:	Shows effect in preview (default)\n" \
												"  Disable Effect:	Hides effect for comparison\n" \
												"  Half Frame:	Shows effect in left half of preview only\n" \
												;

const char* aboutText=	"YUV levels adjuster\n\n" \
												"I wrote this as 'my-first-filter', for fun, and learning c++ & assembler\n" \
												"The reasons I picked this kind of filter were\n" \
												"  Many videos are too dark, and I wanted to be able to boost\n" \
												"  the brightness of dark areas\n" \
												"  I wanted to stay in YUV all the way through the pipline\n" \
												"  Speed, it's a simple task and should be fast\n" \
												"\n" \
												"There are two ways of doing this\n" \
												"  1) Do all the maths on every frame\n" \
												"  2) Work out all the answers beforehand, then just look them up\n" \
												"\n" \
												"The pros/cons to each method are\n" \
												"  1) Slow, uses very little memory\n" \
												"  2) Fast, uses extra memory for lookup tables\n" \
												"\n" \
												"Because we're in YUV space it's possible to use method 2\n" \
												"without too much memory use (128.25K)\n" \
												"In RGB it would be 48Meg of tables - eek\n" \
												"YUV= (Y)256 + (U)256*(V)256*2\n" \
												"RGB= (R)256*(G)256*(B)256*3) bytes\n" \
												"\n" \
												"So what we do is work out all the maths in the configuration dialog\n" \
												"while the sliders are moving etc\n" \
												"\n" \
												"Then when the filter runs, it gets the old pixel values, and looks\n" \
												"in the tables that were worked out earlier for the new values\n" \
												"No maths needed - fast\n" \
												"\n" \
												"Written by Karl Pritchett, dloneranger67@gmail.com\n" ;
///////////////////////////////////////////////////////////////////////////////

class MultiAdjustFilterConfig {
public:
MultiAdjustFilterConfig()
  : MyOptBrightness( 256 ),
  MyOptSaturation( 256 ),
  MyOptContrast( 256 ),
  MyOptHue( 256 ),
  MyOptType( 0 ),
	MyOptLevels239(1),
	MyOptClamp(1),
  MyOptLinkContrast( 1 ),MyOptPreviewMode(cEffectEnabled),
	//MyOptAlwaysOperate( 1 ),
	MyOptHiWidth( MyOptHiWidth_default ),
	MyOptHiReduction( MyOptHiReduction_default ),MyOptPreReduction( 1 ),MyOptPreLevel( MyOptPreLevel_default ),MyOptSatRedsOnly( 0 )
                    {}


public:

sint32        MyOptBrightness;
sint32        MyOptSaturation;
sint32        MyOptContrast;
sint32        MyOptHue;
sint32        MyOptType;
sint32        MyOptCPU;

sint32				MyOptLevels239;
sint32				MyOptClamp;
sint32        MyOptLinkContrast;

sint32 MyOptPreLevel;

sint32				MyOptHiWidth;
sint32				MyOptHiReduction;
sint32				MyOptPreReduction;

sint32				MyOptSatRedsOnly;

//sint32				MyOptAlwaysOperate;

YUV_TableDefs YUV_Lookups;

sint32 MyOptPreviewMode;

void RedoTables();

static void GenTableYUV( YUV_TableDefs *LookupTable,
												 int            _Brightness,			int		_Saturation,
                         int            _Contrast,				int		_Hue,
                         int            _BrightnessType,	bool	_LevelsIn239,
												 bool						_LevelsOut239,		bool	_LinkContrast ,int _Prelevel,
												 bool _PreReduction, int _ReductWidth, int _Reduct, bool SatRedsOnly
												 );
};

void MultiAdjustFilterConfig::RedoTables()
{
	bool lnk	= MyOptLinkContrast == 1;
	bool lvls	= MyOptLevels239		== 1;
	bool clmp	= MyOptClamp				== 1;
	bool alw= true;//MyOptAlwaysOperate==1;

	bool pre = MyOptPreReduction ==1;
	bool satred= MyOptSatRedsOnly==1;

	if (MyOptPreviewMode != cEffectDisabled)
		GenTableYUV( &YUV_Lookups, MyOptBrightness, MyOptSaturation, MyOptContrast, MyOptHue, 
		MyOptType,lvls,clmp,lnk,MyOptPreLevel,pre,MyOptHiWidth,MyOptHiReduction,satred);
	else
		GenTableYUV( &YUV_Lookups, 256, 256, 256, 256, 0,lvls,clmp,lnk,MyOptPreLevel_default,true,MyOptHiWidth_default,0,false);
}


//using asmlib instead
inline signed int Round_float(Tfloat yTemp)
{
	return (int)(yTemp+0.5);
}


inline Tfloat YUV_PreReduce(Tfloat &yTemp, bool _PreReduction, Tfloat &Reduct, int ReductWidth)
{
	if ( _PreReduction && Reduct > 0.0 ) 
		return(  yTemp*(  1.0-( Reduct* ( pow( yTemp,ReductWidth ) ))) );
	else
		return yTemp;
}

inline Tfloat YUV_PostReduce(Tfloat &yTemp, bool _PreReduction, Tfloat &Reduct, int ReductWidth)
{
	if ( !_PreReduction && ( Reduct>0.0 ) )
		{ if (yTemp>1.0) yTemp=1.0;	
			return (  yTemp*(  1.0-( Reduct* ( pow( yTemp,ReductWidth ) ))));
		}
	else
		return ( yTemp );
}

void MultiAdjustFilterConfig::GenTableYUV( YUV_TableDefs *LookupTable,
																					int            _Brightness,			int		_Saturation,
																					int            _Contrast,				int		_Hue,
																					int            _BrightnessType,	bool	_LevelsIn239,
																					bool						_LevelsOut239,		bool	_LinkContrast,
																					int _Prelevel,bool _PreReduction, int _ReductWidth, 
																					int _Reduct, bool SatRedsOnly )
{
	bool _AlwaysOperate=true;

	Tfloat PostLevelMax= MyOptPreLevel_default ;
	PostLevelMax=_Prelevel / PostLevelMax;

	signed int Y;
	signed int U;
	signed int V;
	signed int Yval;
	Tfloat Contrast_Adjust, Saturation_Adjust, Brightness_Adjust, Hue_Adjust, HueCos, HueSin, ConSat;

	Tfloat Reduct = _Reduct / 256.0; //0->0.5
	int ReductWidth = _ReductWidth +1;


	_Hue        = _Hue - 256;					//-256 -> 256
	_Brightness = _Brightness - 256;	//-256 -> 256

	Hue_Adjust				= ( ( (Tfloat)_Hue )	      * FLOAT_256th_Constant ) * (pi / 2);	// -180deg -> 180 deg, def 0
	Saturation_Adjust = ( (Tfloat)  _Saturation ) * FLOAT_256th_Constant;								// 0 - 2, def 1
	Brightness_Adjust = (Tfloat)    _Brightness;																					// -256 - 256 def 0
	Contrast_Adjust		= ( (Tfloat)  _Contrast )		* FLOAT_256th_Constant;								// 0 - 2, def 1

	Tfloat CLinkedContrast = _LinkContrast ? Contrast_Adjust : 1.0;


	HueCos    = cos( Hue_Adjust );
	HueSin    = sin( Hue_Adjust );
	ConSat		= CLinkedContrast * Saturation_Adjust;										//contrast * saturation

	Tfloat Brightness256th = Brightness_Adjust * FLOAT_256th_Constant; // brightness -1 -> 1

	LookupTable->AffectsY	 = (_Brightness == 0) && (_Contrast == 256) ? false : true;

	LookupTable->AffectsUV = (	(_Contrast == 256) || (_LinkContrast == false) ) 
		&& (_Saturation == 256) && (_Hue == 0) ? false : true;

	if ( _AlwaysOperate )
	{ LookupTable->AffectsY=true;
	LookupTable->AffectsUV=true;
	}

	Tfloat BrightnessHalf256th			= (Brightness256th * 0.5);		//(B2 / 2.0);
	Tfloat BrightnessQuarter256th		= (Brightness256th * 0.25);		//(B/4.0);

	int YMin=0;  int YMax=255;
	int UVMin=0; int UVMax=255;
	if (_LevelsOut239)
		{	YMin=16;	YMax=235;
			UVMin=16; UVMax=240;
		};

	Tfloat Y_Divisor;
	Tfloat UV_Divisor;
	Tfloat YUV_Subtract;
	Tfloat Y_Multiplier;
	Tfloat UV_Multiplier;
	Tfloat YUV_Add;

	if (_LevelsIn239)
		{Y_Divisor=235; UV_Divisor=240; YUV_Subtract=16;}
		else
		{Y_Divisor=255; UV_Divisor=255; YUV_Subtract=0;}

	if (_LevelsOut239)
		{Y_Multiplier=235; UV_Multiplier=240; YUV_Add=16;}
		else
		{Y_Multiplier=255; UV_Multiplier=255; YUV_Add=0;}



	for ( Y = 0; Y < 256; Y++ )
	{ 
		Tfloat Y_LevelAdj		= Y - YUV_Subtract; 

		Tfloat Y_Normalized = Y_LevelAdj /(  Y_Divisor -YUV_Subtract );

		Y_Normalized				= Y_Normalized;//*PreLevelMax;

		Tfloat yTemp				= Y_Normalized;

		yTemp=YUV_PreReduce(yTemp, _PreReduction, Reduct, ReductWidth);
		yTemp=yTemp*Contrast_Adjust;

		switch ( _BrightnessType )
		{
			case 0:{//low
				yTemp			= ( yTemp + ( Brightness256th * yTemp * ( -log( yTemp ) ) )) ;
				break;
						 }

			case 1:{//very low
				Tfloat typi	=	yTemp * halfpi;
				typi				= sin(typi) * cos(typi);
				Tfloat mx		= 0.5 * ( 1.0 - (Y_LevelAdj * FLOAT_256th_Constant ) ); // scale down
				yTemp				= (yTemp + ( (Brightness256th * mx * typi ) )) ;
				break;
						 }

			case 2:{//very low boost
				Tfloat typi	= sin(yTemp * pi) * cos(yTemp * halfpi);
				Tfloat mx		= 0.5 * ( 1.0 - (Y_LevelAdj * FLOAT_256th_Constant ) ); // scale down
				yTemp				= ( yTemp + ( Brightness256th * mx * yTemp  ));
				break;
						 }

			case 3:{//flatten
				Tfloat typi	= yTemp*halfpi;
				typi				= sin(typi) * cos(typi);
				yTemp				= (yTemp + (BrightnessHalf256th * typi) );
				break;
						 }

			case 4:{//flatten boost
				Tfloat typi	= sin(yTemp * pi) * cos(yTemp * halfpi);
				yTemp				= (yTemp + (BrightnessHalf256th * typi) ) ;
				break;
						 }

			default:
				yTemp = ( yTemp + BrightnessQuarter256th) ;	

		};
		yTemp=yTemp*PostLevelMax;

		if (yTemp<0.0) yTemp=0.0;			
		else
			if (yTemp>1.0)	yTemp=1.0;

		yTemp	= YUV_PostReduce(yTemp, _PreReduction, Reduct, ReductWidth);

		yTemp = (yTemp * Y_Multiplier)+YUV_Add;

		if (yTemp< YUV_Add)
			yTemp=YUV_Add;

		Yval = Round( yTemp ); //Yval + Round(CLevels);

		if ( Yval < YMin ) 
			LookupTable->Y_Table[Y] = YMin;
		else 
			LookupTable->Y_Table[Y] = Yval > (YMax) ? YMax : (uint8)(Yval);
	}





	//(LookupTable->AffectsUV)
	for ( U = 0; U < 256; U++ )
	{ 
		Tfloat UCentered = (Tfloat)(U - 128);
		UCentered= UCentered * PostLevelMax ;

		for ( V = 0; V < 256; V++ )
		{ 
			Tfloat VCentered = (Tfloat)(V - 128);
			VCentered = VCentered * PostLevelMax;

				Tfloat UTemp = (UCentered * HueCos) + (VCentered * HueSin);
				Tfloat VTemp = (VCentered * HueCos) - (UCentered * HueSin);

				Tfloat Temp_ContrastSat;
				if (SatRedsOnly)
				{
					if ( ( UCentered<0.0 ) && ( VCentered>0.0 ) )  //just reddy stuff
						Temp_ContrastSat=ConSat;				
					else
						Temp_ContrastSat=CLinkedContrast;				
				}
				else
					Temp_ContrastSat=ConSat;



				
				Tfloat UTemp2 = UTemp  * Temp_ContrastSat  ;
				Tfloat VTemp2 = VTemp  * Temp_ContrastSat  ;

				int uAdj= Round ( UTemp2 )+128;
				int vAdj= Round ( VTemp2 )+128;

				if (uAdj < UVMin)	LookupTable->U_Table[U][V] = UVMin;
				else 
					LookupTable->U_Table[U][V] = uAdj > UVMax ? UVMax : (ArrayVals)uAdj;
				if (vAdj < UVMin)	LookupTable->V_Table[U][V] =UVMin;
				else 
					LookupTable->V_Table[U][V] = vAdj > UVMax ? UVMax : (ArrayVals)vAdj;
		}
	}
} // GenTableYUV


///////////////////////////////////////////////////////////////////////////////

class MultiAdjustFilterDialog : public VDXVideoFilterDialog {
public:
MultiAdjustFilterDialog( MultiAdjustFilterConfig &config,
                         IVDXFilterPreview       *ifp ) : mConfig( config ), mpPreview( ifp )
                    {}


bool Show( HWND parent )
{
  return ( 0 != VDXVideoFilterDialog::Show( NULL, MAKEINTRESOURCEA( IDD_FILTER_MULTIADJUST ), parent ) );
}


virtual INT_PTR DlgProc( UINT msg, WPARAM wParam, LPARAM lParam );
void SaveConfigAndRedoFrame();

void ResetDialogAndRedoFrame();



protected:
bool OnInit();

bool OnCommand( int cmd );

void OnDestroy();

void LoadFromConfig();

bool SaveToConfig();

MultiAdjustFilterConfig &mConfig;
MultiAdjustFilterConfig  mOldConfig;
IVDXFilterPreview*const  mpPreview;
};

void MultiAdjustFilterDialog::ResetDialogAndRedoFrame()
{
	LoadFromConfig();
	mConfig.RedoTables();
	if ( mpPreview )
		mpPreview->RedoFrame();
}
void MultiAdjustFilterDialog::SaveConfigAndRedoFrame()
{
	SaveToConfig();
	//if ( SaveToConfig() )
	{ LoadFromConfig(  );
		mConfig.RedoTables();

		if ( mpPreview )
			mpPreview->RedoFrame();
	}
}

INT_PTR MultiAdjustFilterDialog::DlgProc( UINT msg, WPARAM wParam, LPARAM lParam )
{
  switch ( msg )
         {
					 case WM_INITDIALOG:
						 return ( !OnInit() );

					 case WM_DESTROY:
						 OnDestroy();
						 break;

					 case WM_COMMAND:
						 if (LOWORD( wParam ) == IDC_PREVIEW_TYPE )
						 {
								if (HIWORD (wParam)==CBN_SELENDOK)
								{
									int prev = SendMessage ((HWND) lParam,CB_GETCURSEL,0,0);
									mConfig.MyOptPreviewMode=prev;
									SaveConfigAndRedoFrame();
								};
						 }
						 else


						 if (LOWORD( wParam ) == IDC_RESET_PRELEVEL  )
						 {	mConfig.MyOptPreLevel =MyOptPreLevel_default; ResetDialogAndRedoFrame();	SaveConfigAndRedoFrame();}
						 else

						 if (LOWORD( wParam ) == IDC_RESET_HI_Reduction  )
						 {	mConfig.MyOptHiReduction =MyOptHiReduction_default; ResetDialogAndRedoFrame();	SaveConfigAndRedoFrame();}
						 else

						 if (LOWORD( wParam ) == IDC_RESET_Hi_Width )
						 {	mConfig.MyOptHiWidth =MyOptHiWidth_default; ResetDialogAndRedoFrame();	SaveConfigAndRedoFrame();}
						 else

						 if (LOWORD( wParam ) == IDC_RESET_BTYPE )
						 {	mConfig.MyOptType =0; ResetDialogAndRedoFrame();	SaveConfigAndRedoFrame();}
						 else

						 if (LOWORD( wParam ) == IDC_RESET_BRIGHTNESS )
						 {	mConfig.MyOptBrightness =256; ResetDialogAndRedoFrame();	SaveConfigAndRedoFrame();}
						 else

						 if (LOWORD( wParam ) == IDC_RESET_CONTRAST )
						 {	mConfig.MyOptContrast =256; ResetDialogAndRedoFrame();	SaveConfigAndRedoFrame();}
						 else

						 if (LOWORD( wParam ) == IDC_RESET_SATURATION )
						 {	mConfig.MyOptSaturation =256; ResetDialogAndRedoFrame();	SaveConfigAndRedoFrame();}
						 else

						 if (LOWORD( wParam ) == IDC_RESET_HUE )
						 {	mConfig.MyOptHue =256; ResetDialogAndRedoFrame();	SaveConfigAndRedoFrame();}
						 else

						 if ( 
								 ((LOWORD( wParam )		== IDC_BL_LINKCONTRAST) 
								 ||(LOWORD( wParam )	== IDC_BL_ALWAYSOPERATE) 
								 ||(LOWORD( wParam )	== IDC_BL_Levels16_239) 
								 ||(LOWORD( wParam )	== IDC_BL_Clamp16_240) 
								 ||(LOWORD( wParam )	== IDC_BL_Reset)
								 ||(LOWORD( wParam )	== IDC_SATJUSTRED)
								 ||(LOWORD( wParam )	== IDC_BL_PREReduction)
								 )
								 && (HIWORD( wParam ) == BN_CLICKED) 
								 )
								{
									if ((LOWORD( wParam ) == IDC_BL_Reset))
									{
										mConfig.MyOptType				=0;
										mConfig.MyOptBrightness =256;
										mConfig.MyOptContrast		=256;
										mConfig.MyOptHue				=256;
										mConfig.MyOptSaturation =256;
										mConfig.MyOptHiReduction = MyOptHiReduction_default;
										mConfig.MyOptHiWidth = MyOptHiWidth_default;
										mConfig.MyOptPreReduction=1;
										mConfig.MyOptPreLevel = MyOptPreLevel_default;
										mConfig.MyOptSatRedsOnly = 0;
										ResetDialogAndRedoFrame();
									};

									SaveConfigAndRedoFrame();

								return (TRUE);
								}
						 else
								 {
								 if ( OnCommand( LOWORD( wParam ) ) )
										return (TRUE);                  
								 }

						 break;

					 case WM_HSCROLL:

						 if ( SaveToConfig() )
								{
									mConfig.RedoTables();
									if ( mpPreview )
										mpPreview->RedoFrame();
								}

						 return (TRUE);
         }

  return (FALSE);
} // DlgProc

void _SetTickFreq(HWND DialogHandle,int ControlID, int Freq)
{
	HWND hWnd = GetDlgItem( DialogHandle, ControlID );
	if (hWnd!=0)
	SendMessage( hWnd, TBM_SETTICFREQ, Freq, 0 );
}

void _SetSliderRange(HWND DialogHandle,int ControlID, int Range)
{
	SendDlgItemMessage( DialogHandle, ControlID	, TBM_SETRANGE, TRUE, MAKELONG( 0, Range ) );
}

void _SetButtonImage(HWND DialogHandle,int ControlID, HICON Icon)
{
	SendDlgItemMessage( DialogHandle, ControlID	,BM_SETIMAGE ,IMAGE_ICON , (LPARAM)Icon );
}


bool MultiAdjustFilterDialog::OnInit()
{
  mOldConfig = mConfig;

	if (iconReset==0)
		{
			HMODULE  hMod = HINST_THISCOMPONENT;// GetModuleHandle( "MultiAdjust.vdf" );
			if (hMod!=0)
				iconReset		=  (HICON)LoadImage(hMod, MAKEINTRESOURCE(IDI_ICON_RESET),IMAGE_ICON ,16,16,0);
		};

	if (iconReset!=0)
	{	
		_SetButtonImage( mhdlg,IDC_RESET_PRELEVEL,iconReset );
		_SetButtonImage( mhdlg,IDC_RESET_HI_Reduction,iconReset );
		_SetButtonImage( mhdlg,IDC_RESET_Hi_Width,iconReset );
		_SetButtonImage( mhdlg,IDC_RESET_BTYPE,iconReset );
		_SetButtonImage( mhdlg,IDC_RESET_BRIGHTNESS,iconReset );
		_SetButtonImage( mhdlg,IDC_RESET_CONTRAST,iconReset );
		_SetButtonImage( mhdlg,IDC_RESET_SATURATION,iconReset );
		_SetButtonImage( mhdlg,IDC_RESET_HUE,iconReset );
	};


	mConfig.MyOptPreviewMode = cEffectEnabled;
	SendDlgItemMessage( mhdlg, IDC_PREVIEW_TYPE	, CB_RESETCONTENT ,0,0);
	SendDlgItemMessage( mhdlg, IDC_PREVIEW_TYPE	, CB_ADDSTRING , 0,(LPARAM) ComboTxtEffDisabled );
	SendDlgItemMessage( mhdlg, IDC_PREVIEW_TYPE	, CB_ADDSTRING , 0,(LPARAM) ComboTxtEffEnabled );
	SendDlgItemMessage( mhdlg, IDC_PREVIEW_TYPE	, CB_ADDSTRING , 0,(LPARAM) ComboTxtEffHalf );
	SendDlgItemMessage( mhdlg, IDC_PREVIEW_TYPE	, CB_SETCURSEL ,(WPARAM)cEffectEnabled,0);

	_SetSliderRange(mhdlg,IDC_BL_PRELEVEL,256);
	_SetSliderRange(mhdlg,IDC_BL_BRIGHTNESSTYPE,5);
	_SetSliderRange(mhdlg,IDC_BL_BRIGHTNESS,512);
	_SetSliderRange(mhdlg,IDC_BL_CONTRAST,512);
	_SetSliderRange(mhdlg,IDC_BL_SATURATION,512);
	_SetSliderRange(mhdlg,IDC_BL_HUE,512);
	_SetSliderRange(mhdlg,IDC_BL_CPUTYPE,NumberOfCpuMethods - 1);

	_SetSliderRange(mhdlg,IDC_BL_HI_Reduction ,128);
	_SetSliderRange(mhdlg,IDC_BL_HI_Width  ,16);
  SendDlgItemMessage( mhdlg, IDC_BL_PREReduction		, BM_SETCHECK,  BST_CHECKED, 0 );


  SendDlgItemMessage( mhdlg, IDC_BL_Levels16_239		, BM_SETCHECK,  BST_CHECKED, 0 );
  SendDlgItemMessage( mhdlg, IDC_BL_Clamp16_240			, BM_SETCHECK,  BST_CHECKED, 0 );
  SendDlgItemMessage( mhdlg, IDC_BL_LINKCONTRAST		, BM_SETCHECK,  BST_CHECKED, 0 );
  SendDlgItemMessage( mhdlg, IDC_BL_ALWAYSOPERATE		, BM_SETCHECK,  BST_UNCHECKED, 0 );
  SendDlgItemMessage( mhdlg, IDC_SATJUSTRED		, BM_SETCHECK,  BST_UNCHECKED, 0 );

		_SetTickFreq(mhdlg,IDC_BL_HI_Reduction ,16 );
		_SetTickFreq(mhdlg,IDC_BL_HI_Width ,1 );
		

	_SetTickFreq(mhdlg,IDC_BL_PRELEVEL,64);
	_SetTickFreq(mhdlg,IDC_BL_BRIGHTNESS,128);
	_SetTickFreq(mhdlg,IDC_BL_SATURATION,128);
	_SetTickFreq(mhdlg,IDC_BL_CONTRAST,128);
	_SetTickFreq(mhdlg,IDC_BL_HUE,128);
	_SetTickFreq(mhdlg,IDC_BL_BRIGHTNESSTYPE,1);
	_SetTickFreq(mhdlg,IDC_BL_CPUTYPE,1);

  LoadFromConfig();

  HWND hwndFirst = GetDlgItem( mhdlg, IDC_BL_BRIGHTNESS );

  if ( hwndFirst )
     SendMessage( mhdlg, WM_NEXTDLGCTL, (WPARAM)hwndFirst, TRUE );

  //HWND hwndPreview = GetDlgItem( mhdlg, IDC_PREVIEW );

  if ( /*hwndPreview && */ mpPreview )
     {
     //EnableWindow( hwndPreview, TRUE );
     //mpPreview->InitButton( (VDXHWND)hwndPreview );
				
		 bool prevWindowEnabled=mpPreview->isPreviewEnabled();
		 if ( prevWindowEnabled  )
			 mpPreview->Toggle( (VDXHWND)mhdlg  );
			 

     }

  return (false);
} // OnInit


void MultiAdjustFilterDialog::OnDestroy()
{
	if ( mpPreview )
	  mpPreview->InitButton( NULL );
}


bool MultiAdjustFilterDialog::OnCommand( int cmd )
{
  switch ( cmd )
         {
         case IDOK:
					 mConfig.MyOptPreviewMode=cEffectEnabled;
           //SaveToConfig();
					 SaveConfigAndRedoFrame();
           EndDialog( mhdlg, true );
           return (true);

         case IDCANCEL:
           mConfig = mOldConfig;
           EndDialog( mhdlg, false );
           return (true);

				 case IDC_BL_HELP:
					 MessageBox( mhdlg,helpText,"Help: MultiAdjust",MB_OK );
					 return ( true );

				 case IDC_BL_ABOUT:
					 MessageBox( mhdlg,aboutText,"About: MultiAdjust",MB_OK );
					 return ( true );

				/*
         case IDC_PREVIEW:

           if ( mpPreview )
              {
              mpPreview->Toggle( (VDXHWND)mhdlg );
              }

           return (true);
					 */
         }

  return (false);
} // OnCommand


void MultiAdjustFilterDialog::LoadFromConfig()
{
	SendDlgItemMessage(mhdlg, IDC_BL_BRIGHTNESSTYPE, TBM_SETPOS, TRUE, mConfig.MyOptType);
	SendDlgItemMessage(mhdlg, IDC_BL_BRIGHTNESS, TBM_SETPOS, TRUE, mConfig.MyOptBrightness);
	SendDlgItemMessage(mhdlg, IDC_BL_CONTRAST, TBM_SETPOS, TRUE, mConfig.MyOptContrast);
	SendDlgItemMessage(mhdlg, IDC_BL_SATURATION, TBM_SETPOS, TRUE, mConfig.MyOptSaturation);
	SendDlgItemMessage(mhdlg, IDC_BL_HUE, TBM_SETPOS, TRUE, mConfig.MyOptHue);
	SendDlgItemMessage(mhdlg, IDC_BL_PRELEVEL , TBM_SETPOS, TRUE, mConfig.MyOptPreLevel);

	SendDlgItemMessage(mhdlg, IDC_BL_HI_Reduction, TBM_SETPOS, TRUE, mConfig.MyOptHiReduction);
	SendDlgItemMessage(mhdlg, IDC_BL_HI_Width, TBM_SETPOS, TRUE, mConfig.MyOptHiWidth);

	if ( mConfig.MyOptPreReduction == 1 )
	{
		SendDlgItemMessage( mhdlg, IDC_BL_PREReduction, BM_SETCHECK, BST_CHECKED, 0 );
		SendDlgItemMessage( mhdlg, IDC_BL_PREReduction, WM_SETTEXT, 0, (LPARAM)txtPreEffectOn );
	}
	else
	{
		SendDlgItemMessage( mhdlg, IDC_BL_PREReduction, BM_SETCHECK, BST_UNCHECKED, 0 );
		SendDlgItemMessage( mhdlg, IDC_BL_PREReduction, WM_SETTEXT, 0, (LPARAM)txtPreEffectOff );
	}

	if ( mConfig.MyOptSatRedsOnly==0 )
		SendDlgItemMessage( mhdlg, IDC_SATJUSTRED, BM_SETCHECK, BST_UNCHECKED, 0 );
	else
		SendDlgItemMessage( mhdlg, IDC_SATJUSTRED, BM_SETCHECK, BST_CHECKED, 0 );


	//if ( mConfig.MyOptAlwaysOperate == 1 )
	//	SendDlgItemMessage( mhdlg, IDC_BL_ALWAYSOPERATE, BM_SETCHECK, BST_CHECKED, 0 );
	//else
	//	SendDlgItemMessage( mhdlg, IDC_BL_ALWAYSOPERATE, BM_SETCHECK, BST_UNCHECKED, 0 );

	if ( mConfig.MyOptLevels239 == 1 )
		SendDlgItemMessage( mhdlg, IDC_BL_Levels16_239, BM_SETCHECK, BST_CHECKED, 0 );
	else
		SendDlgItemMessage( mhdlg, IDC_BL_Levels16_239, BM_SETCHECK, BST_UNCHECKED, 0 );

	if ( mConfig.MyOptClamp == 1 )
		SendDlgItemMessage( mhdlg, IDC_BL_Clamp16_240, BM_SETCHECK, BST_CHECKED, 0 );
	else
		SendDlgItemMessage( mhdlg, IDC_BL_Clamp16_240, BM_SETCHECK, BST_UNCHECKED, 0 );

	if ( mConfig.MyOptLinkContrast == 1 )
		SendDlgItemMessage( mhdlg, IDC_BL_LINKCONTRAST, BM_SETCHECK, BST_CHECKED, 0 );
	else
		SendDlgItemMessage( mhdlg, IDC_BL_LINKCONTRAST, BM_SETCHECK, BST_UNCHECKED, 0 );
} // LoadFromConfig


bool MultiAdjustFilterDialog::SaveToConfig()
{
  int OptType       =SendDlgItemMessage( mhdlg, IDC_BL_BRIGHTNESSTYPE,	TBM_GETPOS, 0, 0 );
  int OptBrightness =SendDlgItemMessage( mhdlg, IDC_BL_BRIGHTNESS,			TBM_GETPOS, 0, 0 );
  int OptContrast   =SendDlgItemMessage( mhdlg, IDC_BL_CONTRAST,				TBM_GETPOS, 0, 0 );
  int OptSaturation =SendDlgItemMessage( mhdlg, IDC_BL_SATURATION,			TBM_GETPOS, 0, 0 );
  int OptHue        =SendDlgItemMessage( mhdlg, IDC_BL_HUE,							TBM_GETPOS, 0, 0 );
  int OptCPU        =SendDlgItemMessage( mhdlg, IDC_BL_CPUTYPE,					TBM_GETPOS, 0, 0 );
  int OptLevels     =SendDlgItemMessage( mhdlg, IDC_BL_Levels16_239,		BM_GETCHECK, 0, 0 );
  int OptClamp      =SendDlgItemMessage( mhdlg,	IDC_BL_Clamp16_240,			BM_GETCHECK, 0, 0 );
  int OptLinked     =SendDlgItemMessage( mhdlg, IDC_BL_LINKCONTRAST,		BM_GETCHECK, 0, 0 );
  int OptAlways     =SendDlgItemMessage( mhdlg, IDC_BL_ALWAYSOPERATE,		BM_GETCHECK, 0, 0 );

	int OptPreLevel=SendDlgItemMessage( mhdlg, IDC_BL_PRELEVEL,	TBM_GETPOS, 0, 0 );

	int OptHIRed     =SendDlgItemMessage( mhdlg, IDC_BL_HI_Reduction,		TBM_GETPOS, 0, 0 );
  int OptHiWid     =SendDlgItemMessage( mhdlg, IDC_BL_HI_Width,				TBM_GETPOS, 0, 0 );
  int OptPreRed     =SendDlgItemMessage( mhdlg, IDC_BL_PREReduction,		BM_GETCHECK, 0, 0 );
	int OptSatRedOnly = SendDlgItemMessage( mhdlg, IDC_SATJUSTRED,		BM_GETCHECK, 0, 0 );

  OptPreRed = OptPreRed == BST_CHECKED ? 1 : 0;
	OptSatRedOnly= OptSatRedOnly == BST_CHECKED ? 1 : 0;

  OptLevels = OptLevels == BST_CHECKED ? 1 : 0;
  OptLinked = OptLinked == BST_CHECKED ? 1 : 0;
  OptAlways = OptAlways == BST_CHECKED ? 1 : 0;

  if ( (OptType != mConfig.MyOptType)
      || (OptBrightness != mConfig.MyOptBrightness)
      || (OptContrast != mConfig.MyOptContrast)
      || (OptSaturation != mConfig.MyOptSaturation)
      || (OptHue != mConfig.MyOptHue)
			|| (OptPreLevel != mConfig.MyOptPreLevel)
      //|| (OptAlways != mConfig.MyOptAlwaysOperate)
      || (OptLevels != mConfig.MyOptLevels239)
      || (OptClamp != mConfig.MyOptClamp)
      || (OptLinked != mConfig.MyOptLinkContrast)
      || (OptSatRedOnly != mConfig.MyOptSatRedsOnly)
			|| (OptHIRed != mConfig.MyOptHiReduction)
      || (OptHiWid != mConfig.MyOptHiWidth)
			|| (OptPreRed != mConfig.MyOptPreReduction)

			)
     {
     mConfig.MyOptType					= OptType;
     mConfig.MyOptBrightness		= OptBrightness;
     mConfig.MyOptContrast			= OptContrast;
     mConfig.MyOptSaturation		= OptSaturation;
     mConfig.MyOptHue						= OptHue;
     mConfig.MyOptLevels239			= OptLevels;
		 mConfig.MyOptClamp					= OptClamp;
     mConfig.MyOptLinkContrast	= OptLinked;
		 mConfig.MyOptPreLevel      = OptPreLevel;
		 mConfig.MyOptSatRedsOnly   = OptSatRedOnly;
     //mConfig.MyOptAlwaysOperate	= OptAlways;

			 mConfig.MyOptHiReduction=OptHIRed;
      mConfig.MyOptHiWidth=OptHiWid;
			mConfig.MyOptPreReduction=OptPreRed;


     return (true);
     }

  return (false);
} // SaveToConfig


///////////////////////////////////////////////////////////////////////////////

class MultiAdjustFilter : public VDXVideoFilter {
public:
virtual uint32 GetParams();


virtual void Start();

virtual void Run();

virtual bool Configure( VDXHWND hwnd );

virtual void GetSettingString( char *buf, int maxlen );

virtual void GetScriptString( char *buf, int maxlen );

VDXVF_DECLARE_SCRIPT_METHODS();

virtual bool HasSSE2()
{
	return (ff->getCPUFlags() & CPUF_SUPPORTS_SSE2) > 0 ? (true) : (false);
}


protected:

void ScriptConfig7( IVDXScriptInterpreter *isi, const VDXScriptValue  *argv, int argc );


MultiAdjustFilterConfig mConfig;
};

VDXVF_BEGIN_SCRIPT_METHODS( MultiAdjustFilter )
VDXVF_DEFINE_SCRIPT_METHOD( MultiAdjustFilter, ScriptConfig7, "iiiiiiii" )
VDXVF_DEFINE_SCRIPT_METHOD( MultiAdjustFilter, ScriptConfig7, "iiiiiiiii" )
VDXVF_DEFINE_SCRIPT_METHOD( MultiAdjustFilter, ScriptConfig7, "iiiiiiiiiiii" )
VDXVF_DEFINE_SCRIPT_METHOD( MultiAdjustFilter, ScriptConfig7, "iiiiiiiiiiiii" )
VDXVF_END_SCRIPT_METHODS()

uint32 MultiAdjustFilter::GetParams()
{
  switch ( fa->src.mpPixmapLayout->format )
         {
         case nsVDXPixmap::kPixFormat_Y8:
           return (FILTERPARAM_ALIGN_SCANLINES | FILTERPARAM_PURE_TRANSFORM); //| FILTERPARAM_SWAP_BUFFERS
           break;

         case nsVDXPixmap::kPixFormat_YUV422_UYVY:
         case nsVDXPixmap::kPixFormat_YUV422_YUYV:
           return (FILTERPARAM_ALIGN_SCANLINES | FILTERPARAM_PURE_TRANSFORM); //| FILTERPARAM_SWAP_BUFFERS
           break;

         case nsVDXPixmap::kPixFormat_YUV444_Planar:
         case nsVDXPixmap::kPixFormat_YUV422_Planar:
         case nsVDXPixmap::kPixFormat_YUV420_Planar:
         case nsVDXPixmap::kPixFormat_YUV411_Planar:
         case nsVDXPixmap::kPixFormat_YUV410_Planar:
           return (FILTERPARAM_ALIGN_SCANLINES | FILTERPARAM_PURE_TRANSFORM); //| FILTERPARAM_SWAP_BUFFERS
           break;

         default:
           return (FILTERPARAM_NOT_SUPPORTED);
         }

  fa->dst.offset = fa->src.offset;

  return (FILTERPARAM_SUPPORTS_ALTFORMATS);
} // GetParams


void MultiAdjustFilter::Start()
{
  mConfig.RedoTables();
}


void MultiAdjustFilter::Run()
{
  const VDXPixmap &pxdst = *fa->dst.mpPixmap;
  const VDXPixmap &pxsrc = *fa->src.mpPixmap;

	if ( 
			( mConfig.YUV_Lookups.AffectsUV==false) && (mConfig.YUV_Lookups.AffectsY==false )
			&&
			( (uint32)pxdst.data == (uint32)pxsrc.data )
			)	
			return;

  //0 if sse2, else 1
	int UseCpu = HasSSE2() ? 0 : 1;

	bool OnlyLeftHalf = mConfig.MyOptPreviewMode ==cEffectHalf;

  switch ( pxdst.format )
         {
         case nsVDXPixmap::kPixFormat_Y8:
						//only do yplane
						mConfig.YUV_Lookups.AffectsUV = false; //make sure no adjustments get through
						brightcont_run_transYUVPlanar( pxsrc, pxdst, 0, 0, &mConfig.YUV_Lookups, UseCpu,OnlyLeftHalf );
						break;

         case nsVDXPixmap::kPixFormat_YUV422_UYVY:
						brightcont_run_transUYVYInline( pxsrc, pxdst, &mConfig.YUV_Lookups, UseCpu,OnlyLeftHalf );
						break;

         case nsVDXPixmap::kPixFormat_YUV422_YUYV:
						brightcont_run_transYUYVInline( pxsrc, pxdst, &mConfig.YUV_Lookups, UseCpu,OnlyLeftHalf );
						break;

         case nsVDXPixmap::kPixFormat_YUV444_Planar:
						brightcont_run_transYUVPlanar( pxsrc,
																					 pxdst,
                                           _Plane_FullSize,
                                           _Plane_FullSize,
                                           &mConfig.YUV_Lookups,
                                           UseCpu,OnlyLeftHalf );
						break;

         case nsVDXPixmap::kPixFormat_YUV422_Planar:
           brightcont_run_transYUVPlanar( pxsrc,
                                          pxdst,
                                          _Plane_FullSize,
                                          _Plane_HalfSize,
                                          &mConfig.YUV_Lookups,
                                          UseCpu,OnlyLeftHalf );
           break;

         case nsVDXPixmap::kPixFormat_YUV411_Planar:
           brightcont_run_transYUVPlanar( pxsrc,
                                          pxdst,
                                          _Plane_FullSize,
                                          _Plane_QuarterSize,
                                          &mConfig.YUV_Lookups,
                                          UseCpu ,OnlyLeftHalf);
           break;

         case nsVDXPixmap::kPixFormat_YUV420_Planar:
           brightcont_run_transYUVPlanar( pxsrc,
                                          pxdst,
                                          _Plane_HalfSize,
                                          _Plane_HalfSize,
                                          &mConfig.YUV_Lookups,
                                          UseCpu ,OnlyLeftHalf);
           break;

         case nsVDXPixmap::kPixFormat_YUV410_Planar:
           brightcont_run_transYUVPlanar( pxsrc,
                                          pxdst,
                                          _Plane_QuarterSize,
                                          _Plane_QuarterSize,
                                          &mConfig.YUV_Lookups,
                                          UseCpu,OnlyLeftHalf );
           break;
         } //end switch

} // Run


bool MultiAdjustFilter::Configure( VDXHWND hwnd )
{
  MultiAdjustFilterDialog dlg( mConfig, fa->ifp );


  return ( dlg.Show( (HWND)hwnd ) );
}


void MultiAdjustFilter::GetSettingString( char *buf, int maxlen )
{
  SafePrintf( buf, maxlen, " (bri %d%%, sat %d%%, con %d%%, hue %d%%)",
              (mConfig.MyOptBrightness - 256) / 256,
              (mConfig.MyOptSaturation - 256) / 256,
              (mConfig.MyOptContrast - 256) / 256,
              (mConfig.MyOptHue - 256) / 256 );
}


void MultiAdjustFilter::GetScriptString( char *buf, int maxlen )
{
  SafePrintf( buf, maxlen, "Config(%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d)",
              mConfig.MyOptBrightness, mConfig.MyOptSaturation,
              mConfig.MyOptContrast,
              mConfig.MyOptHue, mConfig.MyOptType,
							mConfig.MyOptLevels239,
							mConfig.MyOptClamp,
							mConfig.MyOptLinkContrast,mConfig.MyOptPreLevel,
							mConfig.MyOptHiReduction,mConfig.MyOptHiWidth,mConfig.MyOptPreReduction,
							mConfig.MyOptSatRedsOnly
							);
							
}


void MultiAdjustFilter::ScriptConfig7( IVDXScriptInterpreter *isi, const VDXScriptValue  *argv, int argc )
{
  mConfig.MyOptBrightness   = argv[0].asInt();
  mConfig.MyOptSaturation   = argv[1].asInt();
  mConfig.MyOptContrast     = argv[2].asInt();
  mConfig.MyOptHue          = argv[3].asInt();
  mConfig.MyOptType         = argv[4].asInt();
	int Levels								= argv[5].asInt();
	int Clamp 								= argv[6].asInt();
	int Linked								= argv[7].asInt();
  mConfig.MyOptLevels239		= Levels == 0 ? 0 : 1;   
  mConfig.MyOptClamp				= Clamp  == 0 ? 0 : 1;   
	mConfig.MyOptLinkContrast = Linked == 0 ? 0 : 1;
	if (argc>8)
		mConfig.MyOptPreLevel = argv[8].asInt();
	else
		mConfig.MyOptPreLevel=MyOptPreLevel_default;

	if (argc>9)
	{
		mConfig.MyOptHiReduction = argv[9].asInt();
		mConfig.MyOptHiWidth = argv[10].asInt();
		int hiP = argv[11].asInt();
		mConfig.MyOptPreReduction=  hiP == 0 ? 0 : 1; 
	}
	else
	{
		mConfig.MyOptPreReduction=1;
		mConfig.MyOptHiReduction= MyOptHiReduction_default;
		mConfig.MyOptHiWidth =MyOptHiWidth_default;
	}

	if (argc>12)
		mConfig.MyOptSatRedsOnly= argv[12].asInt(  );
	else
		mConfig.MyOptSatRedsOnly= 0;


}


///////////////////////////////////////////////////////////////////////////////

extern VDXFilterDefinition filterDef_MultiAdjust =
  VDXVideoFilterDefinition<MultiAdjustFilter>(
    "Dloneranger", "MultiAdjust",
    "YUV* Adjust Brightness, Contrast, Hue and Saturation V0.1.7" );